---+-------------------------------------------------------------------------------------------------+
--|																									|
--|  File:			onalib.vhd		                                                          		|	
--|																									|
--|  Project:		onalib																			|
--|																									|
--|  Description: 	VHDL commodities.                      				                         	|
--|																									|
--+-------------------------------------------------------------------------------------------------+
--|	  			 Component 						|					Description						|
--+-------------------------------------------------------------------------------------------------+
--|	sync(clock, d, q)							| Signal synchronize with FF.						|
--|												| Without reset.									|
--+-------------------------------------------------------------------------------------------------+
--|	sync2(clock, d, q)							| Double sinchronize with FF.						|
--|												| Without reset.									|
--+-------------------------------------------------------------------------------------------------+
--|	syncrsld(clock, reset, load, d, q)			| Signal synchornize with FF.					 	|
--|												| Reset and Load.									|
--+-------------------------------------------------------------------------------------------------+
--|	syncv(size)(clock, d, q)					| Vector synchornize (generic).			 			|
--|												| Reset and Load.									|
--+-------------------------------------------------------------------------------------------------+
--|	decoder3to8(i, o)							| Decoder 3 to 8.  						 			|
--+-------------------------------------------------------------------------------------------------+
--|	pfs(clock, a, y)							| Rising edge pulse.								|
--+-------------------------------------------------------------------------------------------------+
--|																									|
--|  Revision history :																				|
--|  Date 		  Version	Author	Description														|
--|  2005-05-13   R00A00	PAU		First Alfa revision (eng)										|
--|																									|
--|  To do:		 																					|
--|																									|
--+-------------------------------------------------------------------------------------------------+
--+-----------------------------------------------------------------+
--| 																|
--|  Copyright (C) 2005 Peio Azkarate, peio@opencores.org   		| 
--| 																|
--|  This source file may be used and distributed without     		|
--|  restriction provided that this copyright statement is not		|
--|  removed from the file and that any derivative work contains	|
--|  the original copyright notice and the associated disclaimer.	|
--|                                                              	|
--|  This source file is free software; you can redistribute it     |
--|  and/or modify it under the terms of the GNU Lesser General     |
--|  Public License as published by the Free Software Foundation;   |
--|  either version 2.1 of the License, or (at your option) any     |
--|  later version.                                                 |
--| 																|
--|  This source is distributed in the hope that it will be         |
--|  useful, but WITHOUT ANY WARRANTY; without even the implied     |
--|  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR        |
--|  PURPOSE.  See the GNU Lesser General Public License for more   |
--|  details.                                                       |
--| 																|
--|  You should have received a copy of the GNU Lesser General      |
--|  Public License along with this source; if not, download it     |
--|  from http://www.opencores.org/lgpl.shtml                       |
--| 																|
--+-----------------------------------------------------------------+ 


--+-----------------------------------------------------------------------------+
--|							PACKAGE COMPONENTS DECLARATION						|
--+-----------------------------------------------------------------------------+

library ieee;
use ieee.std_logic_1164.all;

package onapackage is

	component sync
	port (

		clk       	: in std_logic;
        d       	: in std_logic;
    	q       	: out std_logic

	);   
	end component;

	component sync2
	port (

		clk       	: in std_logic;
        d       	: in std_logic;
    	q       	: out std_logic

	);   
	end component;

	component syncrsld
	port (

		clk       	: in std_logic;
    	rst       	: in std_logic;
    	load     	: in std_logic;   	
        d       	: in std_logic;
    	q       	: out std_logic

	);   
	end component;

	component syncv
	generic ( size: integer := 8 );
	port (

		clk       	: in std_logic;
        d       	: in std_logic_vector(size-1 downto 0);
    	q       	: out std_logic_vector(size-1 downto 0)

	);   
	end component;

	component decoder3to8
	port (

        i       	: in std_logic_vector(2 downto 0);
    	o       	: out std_logic_vector(7 downto 0)

	);   
	end component;

	component pfs
	port (

		clk       	: in std_logic;
        a       	: in std_logic;
    	y       	: out std_logic

	);   
	end component;
			
end onapackage;



--+-----------------------------------------------------------------------------+
--|								ENTITY & ARCHITECTURE							|
--+-----------------------------------------------------------------------------+


--+-----------------------------------------+
--|  sync									|
--+-----------------------------------------+

library ieee;
use ieee.std_logic_1164.all;

entity sync is
port (

	clk       	: in std_logic;
    d       	: in std_logic;
   	q       	: out std_logic

);   
end sync;

architecture rtl of sync is
begin

	SYNCP: process( clk, d )
	begin
	
        if ( rising_edge(clk) ) then
			q <= d;
		end if;
        
	end process SYNCP;

end rtl;


--+-----------------------------------------+
--|  sync2									|
--+-----------------------------------------+

library ieee;
use ieee.std_logic_1164.all;

entity sync2 is
	port (

		clk       	: in std_logic;
        d       	: in std_logic;
    	q       	: out std_logic

	);   
end sync2;

architecture rtl of sync2 is
	signal tmp:	std_logic;
begin

	SYNC2P: process ( clk, d, tmp)
	begin	
	
        if ( rising_edge(clk) ) then 
        	tmp <= d; 
        	q <= tmp;
        end if;
        
	end process SYNC2P;
	
end rtl;


--+-----------------------------------------+
--|  syncrsld								|
--+-----------------------------------------+

library ieee;
use ieee.std_logic_1164.all;

entity syncrsld is
	port (

		clk       	: in std_logic;
		reset		: in std_logic;
		load		: in std_logic;
        d       	: in std_logic;
    	q       	: out std_logic
    	
	);   
end syncrsld;

architecture rtl of syncrsld is
begin

	SYNCRSLDP: process ( reset, clk, load, d)
	begin
	
		if ( reset = '1' ) then
			q <= '0';
        elsif ( rising_edge(clk) ) then         	
        	if ( load = '1' ) then
        		q <= d; 
        	end if;        	
        end if;
        
	end process SYNCRSLDP;
	
end rtl;


--+-----------------------------------------+
--|  syncv									|
--+-----------------------------------------+

library ieee;
use ieee.std_logic_1164.all;

entity syncv is
generic ( size: integer := 8 );
port (

	clk       	: in std_logic;
    d       	: in std_logic_vector(size-1 downto 0);
   	q       	: out std_logic_vector(size-1 downto 0)

);   
end syncv;

architecture rtl of syncv is
begin

	SYNCVP: process( clk, d )
	begin
	
        if ( rising_edge(clk) ) then
			q <= d;
		end if;
        
	end process SYNCVP;

end rtl;


--+-----------------------------------------+
--|  decoder3to8							|
--+-----------------------------------------+

library ieee;
use ieee.std_logic_1164.all;

entity decoder3to8 is
port (

    i       	: in std_logic_vector(2 downto 0);
   	o       	: out std_logic_vector(7 downto 0)

);   
end decoder3to8;

architecture rtl of decoder3to8 is
begin

	DECOD3TO8P: process( i )
	begin
	
        if    ( i = "111" ) then o <= "10000000";
		elsif ( i = "110" ) then o <= "01000000";
		elsif ( i = "101" ) then o <= "00100000";
		elsif ( i = "100" ) then o <= "00010000";
		elsif ( i = "011" ) then o <= "00001000";
		elsif ( i = "010" ) then o <= "00000100";
		elsif ( i = "001" ) then o <= "00000010";
		elsif ( i = "000" ) then o <= "00000001";		
		end if;
        
	end process DECOD3TO8P;

end rtl;


--+-----------------------------------------+
--|  pfs									|
--+-----------------------------------------+

library ieee;
use ieee.std_logic_1164.all;

entity pfs is
port (

	clk       	: in std_logic;
    a       	: in std_logic;
   	y       	: out std_logic

);   
end pfs;

architecture rtl of pfs is

	signal a_s	: std_logic;
	
begin

	SYNCP: process( clk, a )
	begin
	
        if ( rising_edge(clk) ) then
			a_s <= a;
		end if;
        
	end process SYNCP;

	y <= a and (not a_s);
	
end rtl;

